// $Id: doc.C,v 1.12 2003/08/08 11:46:30 dvermeir Exp $

#include "doc.h"
#include "node.h"
#include <xsltwrapp/xsltwrapp.h>
#include <iostream>
#include <fstream>

//////// Auxiliary function.

static void
init() {
static xslt::init init_;
}

//////// Constructors, destructor.

Dv::Xml::Document::Document(): xml::document() {
}

Dv::Xml::Document::Document(const Dv::Xml::Node::Ref& ref) throw (Dv::Xml::Exception):
  xml::document() {
root(ref);
}

Dv::Xml::Document::Document(const std::string& name) 
  throw (Dv::Xml::Exception): xml::document() {
Dv::Xml::Node n(name);
root(n);
}

Dv::Xml::Document::Document(const std::string& name, const std::string& content) 
  throw (Dv::Xml::Exception): xml::document() {
Dv::Xml::Node n(name, content);
root(n);
}

Dv::Xml::Document::Document(const Dv::Util::File& file) 
  throw (Dv::Xml::Exception): xml::document() {
  {
  std::ifstream ifs(file.path());
  if (!ifs)
    throw Dv::Xml::Exception(file.str() + ": cannot open for reading");
  }
parse(file.str());
}

Dv::Xml::Document::Document(const char* data, size_t size)
  throw (Dv::Xml::Exception): xml::document() {
parse(data, size);
}

Dv::Xml::Document::Document(const std::string& stylesheet, Dv::Xml::Document& doc)
    throw (Dv::Xml::Exception): xml::document() {
init();
try {
  static const std::string MSG("Dv::Xml::Document::apply error: ");
  xslt::stylesheet style(stylesheet.c_str());

  // This will throw a std::runtime_error upon failure.
  (*static_cast<xml::document*>(this)) = style.apply(doc);
  }
catch (std::exception& e) {
  throw Dv::Xml::Exception(e.what());
  }
}

Dv::Xml::Document::~Document() {
}

//////// Setting and getting properties.

Dv::Xml::Node::Ref
Dv::Xml::Document::root() const {
return Dv::Xml::Node::Ref(const_cast<Dv::Xml::Document*>(this)->get_root_node().self());
}

Dv::Xml::Document&
Dv::Xml::Document::operator=(const Dv::Xml::Node::Ref& ref) throw (Dv::Xml::Exception) {
root(ref);
return *this;
}

void
Dv::Xml::Document::root(const Dv::Xml::Node::Ref& ref) throw (Dv::Xml::Exception) {
const xml::node* root = static_cast<const xml::node*>(ref);
if (!root)
  throw Dv::Xml::Exception("attempt to assign nil node to root");
set_root_node(*root);
}

//////// Validate
bool
Dv::Xml::Document::valid(const std::string& dtd) {
if ( dtd.size() ) 
  return validate(dtd.c_str());
else
  return validate();
}


//////// Parsing: assign document from text representation.
Dv::Xml::Document&
Dv::Xml::Document::parse(const std::string& filenm) 
  throw (Dv::Xml::Exception) {
init();
try {
  xml::tree_parser parser(filenm.c_str());
  swap(parser.get_document());
  }
catch (std::exception& e) {
  throw Dv::Xml::Exception(e.what());
  }
return *this;
}

Dv::Xml::Document&
Dv::Xml::Document::parse(const char* data, size_t size)
  throw (Dv::Xml::Exception) {
init();
try {
  xml::tree_parser parser(data, size);
  swap(parser.get_document());
  }
catch (std::exception& e) {
  throw Dv::Xml::Exception(e.what());
  }
return *this;
}

//////// Accessing document nodes.
Dv::Xml::Node::Ref
Dv::Xml::Document::select(const std::string& path) const 
  throw (Dv::Xml::Exception) {
try {
  Dv::Xml::Node::Ref ref(root());
  std::string::size_type cur(0);
  std::string::size_type nxt(0);
  while ( (nxt = path.find('/', cur)) != std::string::npos ) {
    std::string segment(path, cur, nxt - cur);
    ref = ref/segment;
    cur = nxt+1;
    if (cur >= path.size() )
      cur = std::string::npos;
    }
  if (cur != std::string::npos) {
    std::string segment(path, cur);
    ref = ref/segment;
    }
  return ref;
  }
catch (std::exception& e) {
  throw Dv::Xml::Exception(path + "\n" + e.what());
  }
}

