// $Id: dviconv.C,v 1.1 2003/05/24 12:28:42 dvermeir Exp $

#include <iconv.h>
#include <errno.h> // errno
#include <string.h> // strlen(), strerror()
#include <dvutil/date.h>
#include <dvutil/tostring.h>
#include <xmlwrapp/xmlwrapp.h>

#include "dviconv.h"

namespace Dv {
namespace Xml {

void
iconv(const char* to_charset, const char* from_charset, 
  std::string& target, const char* source) throw (std::runtime_error) {
iconv_t cd = iconv_open(to_charset, from_charset);
if (cd == reinterpret_cast<iconv_t>(-1) )
  throw std::runtime_error("iconv_open failed");
size_t s_len(strlen(source));
char* s_ptr(const_cast<char*>(source));
size_t max_o_len = s_len * 5;
size_t o_len(max_o_len);
char* const buf = new char[o_len];
char* o_ptr(buf);
::iconv(cd, &s_ptr, &s_len, &o_ptr, &o_len);
if ( (s_len != 0) || errno) {
  int sys_err(errno);
  throw std::runtime_error(std::string(source) + 
    ": Dv::Xml::iconv from " + from_charset + " to " + to_charset + " failed [" +
    (sys_err ? strerror(sys_err) : "") + "]"
    );
  }
target.resize(0);
target.append(buf, max_o_len - o_len);
delete [] buf;
iconv_close(cd);
}

void
iconv(const char* to_charset, const char* from_charset, 
  std::string& target, const std::string& source) throw (std::runtime_error) {
iconv_t cd(::iconv_open(to_charset, from_charset));
if (cd == reinterpret_cast<iconv_t>(-1) )
  throw std::runtime_error("iconv_open failed");
size_t s_len(source.size());
char* s_ptr(const_cast<char*>(source.data()));
size_t max_o_len = s_len * 5;
size_t o_len(max_o_len);
char* const buf = new char[o_len];
char* o_ptr(buf);
::iconv(cd, &s_ptr, &s_len, &o_ptr, &o_len);
if ((s_len != 0) || errno) {
  int sys_err(errno);
  throw std::runtime_error(std::string(source) + 
    ": Dv::Xml::iconv from " + from_charset + " to " + to_charset + " failed [" +
    (sys_err ? strerror(sys_err) : "") + "]"
    );
  }
target.resize(0);
target.append(buf, max_o_len - o_len);
delete [] buf;
iconv_close(cd);
}

std::string
iconv(const char* to_charset, const char* from_charset, const char* source ) 
  throw (std::runtime_error) {
std::string out;
iconv(to_charset, from_charset, out, source);
return out;
}

std::string
iconv(const char* to_charset, const char* from_charset, const std::string& source ) 
  throw (std::runtime_error) {
std::string out;
iconv(to_charset, from_charset, out, source);
return out;
}

}}

