#ifndef DVXML_ICONV_H
#define DVXML_ICONV_H
// $Id: dviconv.h,v 1.1 2003/05/24 12:28:42 dvermeir Exp $

#include <string>
#include <stdexcept>
#include <dvutil/date.h>
#include <xmlwrapp/xmlwrapp.h>

namespace Dv {
namespace Xml {

/** Convert source C-string between character sets.
 * @param to_charset character set to convert to, e.g. "UTF8"
 * @param from_charset character set to convert from, e.g. "ISO-8859-1"
 * @param target string containing new version of source string.
 * @param source C-string
 * @exception std::runtime_error if anything goes wrong.
 * Use iconv --list to see all supported character sets.
 */
void iconv(const char* to_charset, const char* from_charset, 
  std::string& target, const char* source) throw (std::runtime_error);

/** Convert source C-string between character sets.
 * @param to_charset character set to convert to, e.g. "UTF8"
 * @param from_charset character set to convert from, e.g. "ISO-8859-1"
 * @param target string containing new version of source string.
 * @param source C-string
 * @exception std::runtime_error if anything goes wrong.
 * Use iconv --list to see all supported character sets.
 */
void iconv(const char* to_charset, const char* from_charset, 
  std::string& target, const std::string& source) throw (std::runtime_error);

/** Convert source C-string between character sets.
 * @param to_charset character set to convert to, e.g. "UTF8"
 * @param from_charset character set to convert from, e.g. "ISO-8859-1"
 * @param source C-string
 * @return string containing new version of source string (in target character set).
 * @exception std::runtime_error if anything goes wrong.
 * Use iconv --list to see all supported character sets.
 */
std::string iconv(const char* to_charset, const char* from_charset, 
  const char* source) throw (std::runtime_error);

/** Convert source C-string between character sets.
 * @param to_charset character set to convert to, e.g. "UTF8"
 * @param from_charset character set to convert from, e.g. "ISO-8859-1"
 * @param source C-string
 * @return string containing new version of source string (in target character set).
 * @exception std::runtime_error if anything goes wrong.
 * Use iconv --list to see all supported character sets.
 */
std::string iconv(const char* to_charset, const char* from_charset, 
  const std::string& source) throw (std::runtime_error);

/** Convert "ISO-8859-1" string to "UTF8" string.
 * @param source string (using "ISO-8859-1" character set).
 * @return string containing source string (in "UTF8" character set).
 * @exception std::runtime_error if anything goes wrong.
 * Use iconv --list to see all supported character sets.
 */
inline std::string iso2utf8(const std::string& source ) throw (std::runtime_error) {
  return iconv("UTF8", "ISO-8859-1", source); 
  }

/** Convert "ISO-8859-1" C-string to "UTF8" string.
 * @param source string (using "ISO-8859-1" character set).
 * @return string containing source string (in "UTF8" character set).
 * @exception std::runtime_error if anything goes wrong.
 * Use iconv --list to see all supported character sets.
 */
inline std::string iso2utf8(const char* source ) throw (std::runtime_error) {
  return iconv("UTF8", "ISO-8859-1", source); 
  }


}}
#endif

