// $Id: dvxml.C,v 1.4 2003/08/08 11:24:34 dvermeir Exp $
#include <iostream>
#include <string>
#include <dvxml/xml.h>

static const std::string USAGE(
  "dvxml operation arguments\n"
  "with operation one of:\n"
  "  dvxml check dtd-file xmlfile..\n"
  "  dvxml select xmlfile path/from/root\n"
  "  dvxml select xmlfile path/from/root[attrname]\n"
  );

////// Select path argv[1] from xml files argv[2]..
void
select(int argc, char** argv) {
if (argc < 3)
  throw std::runtime_error(USAGE);

std::string path_and_attr(argv[1]);
std::string::size_type lbracket(path_and_attr.find('['));
std::string path(path_and_attr, 0, lbracket);
// std::cerr << "path=" << path << std::endl;
std::string attr;
if (lbracket != std::string::npos) {
  std::string::size_type rbracket(path_and_attr.find(']', lbracket));
  // std::cerr << "lbr=" << lbracket << std::endl;
  // std::cerr << "rbr=" << rbracket << std::endl;
  if (rbracket == std::string::npos)
    throw std::runtime_error("no closing ']' in path");
  attr = std::string(path_and_attr, lbracket + 1, rbracket - lbracket -1);
  // std::cerr << "attr=" << attr << std::endl;
  }

for (int i = 2; (i<argc); ++i) 
  try {
    Dv::Util::File file(argv[i]);
    Dv::Xml::Document doc(file);
    Dv::Xml::Node::Ref ref(doc.select(path));
    if ( attr.size() )
      std::cout << ref(attr) << std::endl;
    else
      std::cout << ref.str() << std::endl;
    }
  catch (Dv::Xml::Exception& e) {
    }
}

////// Check whether argv[2].. conform to dtd file in argv[1]
void
check(int argc, char** argv) {

if (argc < 3)
  throw std::runtime_error(USAGE);

const char* dtd_fn(argv[1]);

static const std::string INVALID("Invalid document");

for (int i = 2; (i<argc); ++i) {
  try {
    Dv::Util::File file(argv[i]);
    Dv::Xml::Document doc(file);
    if (! doc.valid(dtd_fn))
      throw std::runtime_error(INVALID + ": " + argv[i] + " does not conform to " + dtd_fn);
    }
  catch (Dv::Xml::Exception& e) {
    throw std::runtime_error(INVALID + ": " + argv[i] + ": " + e.what() );
    }
  }
}

int
main(int argc, char** argv) {

try {
  if (argc < 2)
    throw std::runtime_error(USAGE);
  std::string command(argv[1]);
  if (command == "check")
    check(argc-1, ++argv);
  else if (command == "select")
    select(argc-1, ++argv);
  else 
    throw std::runtime_error(USAGE);

  return 0;
  }
catch (std::exception& e) {
  std::cerr << e.what() << std::endl;
  return 1;
  }
}
