// $Id: node.C,v 1.16 2003/09/08 14:11:01 dvermeir Exp $
#include "node.h"
#include <iostream>
#include <dvutil/ntostr.h>

//////// Auxiliary assertion function.
void
Dv::Xml::Node::Ref::assert_valid(const std::string& operation) const 
  throw (Dv::Xml::Exception) {
if (nil())
  throw Dv::Xml::Exception(operation + ": nil node");
}

//////// Auxiliary Dv::Xml::Node::AttributeReference class.

Dv::Xml::Node::AttributeReference::AttributeReference(const xml::node::iterator n,
  const std::string& nm): name(nm), node(n) {
}

const std::string&
Dv::Xml::Node::AttributeReference::operator=(const std::string& value) {
// node->get_attributes().insert(name.c_str(), Dv::Xml::to_html(value).c_str());
node->get_attributes().insert(name.c_str(), value.c_str());
return value;
}

std::string
Dv::Xml::Node::AttributeReference::str() const {
xml::attributes::iterator it(node->get_attributes().find(name.c_str()));
if ( it == node->get_attributes().end() ) 
  return "";
else
  return it->get_value();
}

//////// Constructors.

Dv::Xml::Node::Ref::Ref(xml::node& node): it_(node.self()), nil_(false) {
}

Dv::Xml::Node::Ref::Ref(xml::node::iterator it, bool nil): it_(it), nil_(nil) {
}

//////// Retrieve info on Node
std::string
Dv::Xml::Node::Ref::name() const throw (Exception) {
assert_valid("name()");
return it_->get_name();
}

Dv::Xml::Node::Ref::operator const xml::node*() const {
if (nil())
  return 0;
return &*it_;
}

//////// Setting Node properties
Dv::Xml::Node::Ref&
Dv::Xml::Node::Ref::name(const std::string& nm) throw (Exception) {
assert_valid("name(name)");
it_->set_name(nm.c_str());
return *this;
}

//////// Finding and creating children.

Dv::Xml::Node::Ref
Dv::Xml::operator/(const Dv::Xml::Node::Ref& ref, const std::string& nm) 
  throw (Dv::Xml::Exception) {
ref.assert_valid(nm + "/");
xml::node::iterator itc(ref.it_->find(nm.c_str()));
return Dv::Xml::Node::Ref(itc, itc == ref.it_->end() );
}

Dv::Xml::Node::Ref
Dv::Xml::operator>>(const Dv::Xml::Node::Ref& ref, const std::string& name) 
  throw (Dv::Xml::Exception) {
static const std::string op(">>");
ref.assert_valid(op + name);
xml::node n(name.c_str());
return Dv::Xml::Node::Ref(ref.it_->insert(n));
} 

Dv::Xml::Node::Ref
Dv::Xml::operator>>(const Dv::Xml::Node::Ref& ref, const Dv::Xml::Node::Ref& nref) 
  throw (Dv::Xml::Exception) {
ref.assert_valid("BadRef::operator>>(const Ref&)");
nref.assert_valid("operator>>(const BadRef&)");
return Dv::Xml::Node::Ref(ref.it_->insert(*(nref.it_)));
}

//////// Iterator operations

Dv::Xml::Node::Ref
Dv::Xml::Node::Ref::operator++() throw (Exception) {
assert_valid("operator++");
xml::node::iterator parent(it_->parent());
xml::node::iterator next(it_);
it_ = parent->find(it_->get_name(), ++next);
nil_ = (it_ == parent->end());
return *this;
}

Dv::Xml::Node::Ref
Dv::Xml::Node::Ref::end() const throw (Exception) {
assert_valid("end()");
return Ref(it_->end(), true);
}

bool
Dv::Xml::operator==(const Dv::Xml::Node::Ref& n, const Dv::Xml::Node::Ref& m) {
if ( n.nil() )
  return m.nil();
else
  return m.it_ == n.it_;
}

//////// Setting and retrieving attributes.

bool
Dv::Xml::Node::Ref::defined(const std::string& name) const throw (Exception) {
assert_valid(name + "(defined)");
xml::attributes::iterator it(it_->get_attributes().find(name.c_str()));
return ( it != it_->get_attributes().end() );
}

const std::string
Dv::Xml::Node::Ref::operator()(const std::string& nm) const throw (Exception) {
assert_valid(nm + "(get_attr)");
xml::attributes::iterator it(it_->get_attributes().find(nm.c_str()));
if ( it == it_->get_attributes().end() )
  throw Dv::Xml::Exception(std::string(name()) + "." + nm + ": no such attribute");
return it->get_value();
}

Dv::Xml::Node::AttributeReference
Dv::Xml::Node::Ref::operator[](const std::string& name) throw (Exception) {
assert_valid(name + "[]");
return AttributeReference(it_, name);
}

//////// Setting and retrieving text content.

Dv::Xml::Node::Ref::operator std::string() const throw (Exception) {
return str(true);
}

std::string
Dv::Xml::Node::Ref::str(bool trim) const throw (Exception) {
assert_valid("get_content");
if (trim) {
  std::string result(it_->get_content());
  Dv::Util::strim(result);
  return result;
  }
else
  return it_->get_content();
}

Dv::Xml::Node::Ref&
Dv::Xml::Node::Ref::operator=(const std::string& text) throw (Exception) {
assert_valid(text + ": set_content");
// it_->set_content(Dv::Xml::to_html(text).c_str());
it_->set_content(text.c_str());
return *this;
}

//////// Replacing a node.


Dv::Xml::Node::Ref&
Dv::Xml::Node::Ref::replace(const Ref& ref) {
assert_valid("replace");
if (this == &ref)
  return *this;
if ( ref.nil() ) {
  nil_ = ref.nil_;
  it_ = ref.it_;
  }
else { // copy underlying xml::node
  *it_ = *(ref.it_);
  }
return *this;
}

std::ostream&
Dv::Xml::operator<<(std::ostream& os, const Dv::Xml::Node::Ref& ref) {
const xml::node* p(static_cast<const xml::node*>(ref));
if (p)
  return os << *p;
else
  return os << "NIL";
}
